﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.ConditionActionEncoding;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.VariableVector {
  [Item("UniformSomePositionManipulator", "Description missing")]
  [StorableClass]
  public class UniformSomePositionManipulator : VariableVectorManipulator, IProbabilityMutatorOperator {

    #region Parameter Properties
    public IValueLookupParameter<PercentValue> ProbabilityParameter {
      get { return (IValueLookupParameter<PercentValue>)Parameters["Probability"]; }
    }
    public ILookupParameter<IItemSet<VariableVectorAction>> PossibleActionsParameter {
      get { return (ILookupParameter<IItemSet<VariableVectorAction>>)Parameters["PossibleActions"]; }
    }
    #endregion

    [StorableConstructor]
    protected UniformSomePositionManipulator(bool deserializing) : base(deserializing) { }
    protected UniformSomePositionManipulator(UniformSomePositionManipulator original, Cloner cloner)
      : base(original, cloner) {
    }
    public UniformSomePositionManipulator()
      : base() {
      Parameters.Add(new ValueLookupParameter<PercentValue>("Probability", "The probability for each dimension to be manipulated."));
      Parameters.Add(new LookupParameter<IItemSet<VariableVectorAction>>("PossibleActions"));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new UniformSomePositionManipulator(this, cloner);
    }

    protected override VariableVector Manipulate(IRandom random, VariableVectorInput input, VariableVector child) {
      return Manipulate(random, input, child, ProbabilityParameter.ActualValue.Value, PossibleActionsParameter.ActualValue);
    }

    public static VariableVector Manipulate(IRandom random, VariableVectorInput input, VariableVector child, double probability, IItemSet<VariableVectorAction> possibleActions) {
      child = ManipulateCondition(random, input, child, probability);
      return ManipulateAction(random, child, probability, possibleActions);
    }

    public static VariableVector ManipulateCondition(IRandom random, VariableVectorInput input, VariableVector child, double probability) {
      for (int index = 0; index < child.Condition.VirtualLength; index++) {
        if (random.NextDouble() < probability) {
          child = UniformOnePositionInConditionManipulator.Manipulate(random, input, child, index);
        }
      }
      return child;
    }

    public static VariableVector ManipulateAction(IRandom random, VariableVector child, double probability, IItemSet<VariableVectorAction> possibleActions) {
      if (random.NextDouble() < probability) {
        UniformActionManipulator.Manipulate(random, child, possibleActions);
      }
      return child;
    }
  }
}
