﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Optimization.Operators.LCS.DefaultRule {
  [Item("MajorDefaultRule", "Description missing")]
  [StorableClass]
  public class MajorDefaultRule : DefaultRuleOperator, IStochasticOperator {

    #region Parameter Properties
    public ILookupParameter<ItemCollection<IntValue>> ClassCountParameter {
      get { return (ILookupParameter<ItemCollection<IntValue>>)Parameters["ClassCount"]; }
    }
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }
    #endregion

    [StorableConstructor]
    protected MajorDefaultRule(bool deserializing) : base(deserializing) { }
    protected MajorDefaultRule(MajorDefaultRule original, Cloner cloner)
      : base(original, cloner) {
    }
    public MajorDefaultRule()
      : base() {
      Parameters.Add(new LookupParameter<ItemCollection<IntValue>>("ClassCount"));
      Parameters.Add(new LookupParameter<IRandom>("Random"));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MajorDefaultRule(this, cloner);
    }

    public override IOperation Apply() {
      if (NichingParameter.ActualValue == null) {
        NichingParameter.ActualValue = new BoolValue(false);
        NichesParameter.ActualValue = new IntValue(1);
        var max = ClassCountParameter.ActualValue.MaxItems(x => x.Value).First().Value;
        var indices = new List<int>();
        int i = 0;
        foreach (var classCount in ClassCountParameter.ActualValue) {
          if (classCount.Value == max) {
            indices.Add(i);
          }
          i++;
        }
        int index;
        // if there are more than one major classes
        if (indices.Count > 1) {
          int help = RandomParameter.ActualValue.Next(0, indices.Count);
          index = indices[help];
        } else {
          index = indices[0];
        }
        IGAssistNiche niche = GAssistNichesProblemDataParameter.ActualValue.GetPossibleNiches().ElementAt(index);
        DefaultClassParameter.ActualValue = niche;

        foreach (var individual in IndividualParameter.ActualValue) {
          individual.SetNiche(RandomParameter.ActualValue, niche);
        }

        OperationCollection next = new OperationCollection();
        next.Add(EvaluateSubScopes());
        next.Add(base.Apply());
        return next;
      }
      return base.Apply();
    }
  }
}
